/**
 * Vue Number Format 2.2.1
 * (c) 2021-2023 Dipak Sarkar <hello@dipaksarkar.in> (https://dipaksarkar.in/)
 * @license MIT
 */
import Vue from 'vue';

var options$1 = {
    prefix: '',
    suffix: '',
    separator: ',',
    decimal: '.',
    precision: 2,
    minimumFractionDigits: null,
    prefill: true,
    reverseFill: false,
    min: null,
    max: null,
    nullValue: ''
};

function between(min, n, max) {
    return Math.max(min, Math.min(n, max));
}
// Uncaught RangeError: toFixed() digits argument must be between 0 and 20 at Number.toFixed
function fixed(precision) {
    return between(0, precision, 20);
}
function toFixed(numbers, precision) {
    const exp = Math.pow(10, precision);
    const float = parseFloat(numbers) / exp || 0;
    return float.toFixed(fixed(precision));
}
/**
 * Number format class
 * @param {Options} config
 */
class NumberFormat {
    constructor(config) {
        this.options = Object.assign(options$1, config);
        this.input = '';
        this.number = '';
        this.isClean = true;
        const { prefix, suffix, decimal } = this.options;
        this.preSurRegExp = new RegExp(`${prefix}|${suffix}`, 'g');
        this.numberRegExp = new RegExp(`[^0-9\\${decimal}]+`, 'gi');
        this.cleanRegExp = new RegExp('[^0-9]+', 'gi');
        this.negativeRegExp = new RegExp('[^0-9\\-]+', 'gi');
    }
    isNull() {
        if (this.isClean) {
            return !this.numberOnly(this.input, this.cleanRegExp);
        }
        return !this.numberOnly(this.input, this.negativeRegExp);
    }
    clean(clean = false) {
        this.isClean = clean;
        return this;
    }
    sign() {
        if (this.input === null || this.input === undefined) {
            return '';
        }
        const hasMinus = this.input.toString().indexOf('-') >= 0;
        if (this.isClean) {
            return hasMinus && this.realNumber() > 0 ? '-' : '';
        }
        return hasMinus ? '-' : '';
    }
    toNumber(str) {
        return Number(str);
    }
    numberOnly(str, regExp) {
        return str === null || str === void 0 ? void 0 : str.toString().replace(regExp, '');
    }
    isNegative() {
        return this.sign() === '-';
    }
    numbers() {
        if (this.options.reverseFill) {
            this.number = toFixed(this.numberOnly(this.input, /\D+/g), this.options.precision).replace('.', this.options.decimal);
        }
        else if (typeof this.input === 'number') {
            this.number = this.parts(this.input.toString().replace('-', ''), '.').join(this.options.decimal);
        }
        else {
            const input = this.input.replace(this.preSurRegExp, '');
            this.number = this.numberOnly(input, this.numberRegExp);
            this.number = this.parts(this.number).join(this.options.decimal);
        }
        return this.number;
    }
    realNumber() {
        return parseFloat(this.numbers().toString().replace(this.options.decimal, '.'));
    }
    parts(num, separator) {
        const decimal = separator || this.options.decimal;
        let parts = num.toString().split(decimal);
        if (parts.length > 1) {
            parts[0] = this.toNumber(parts[0]) || 0;
            parts[1] = parts.slice(1, parts.length).join('');
            parts = parts.slice(0, 2);
        }
        if (this.isClean) {
            const newNumber = this.toNumber(parts.join('.')).toFixed(this.options.precision);
            const cleanNumber = this.toNumber(newNumber);
            const minimumDigits = cleanNumber.toFixed(this.options.minimumFractionDigits);
            const hasMinFraction = this.options.minimumFractionDigits >= 0 && cleanNumber.toString().length < minimumDigits.length;
            if (hasMinFraction) {
                parts = minimumDigits.toString().split('.');
            }
            else {
                parts = cleanNumber.toString().split('.');
            }
        }
        return parts.slice(0, 2);
    }
    addSeparator() {
        const parts = this.numbers().split(this.options.decimal);
        parts[0] = parts[0].toString().replace(/(\d)(?=(?:\d{3})+\b)/gm, `$1${this.options.separator}`);
        return parts.join(this.options.decimal);
    }
    /**
     * Format the input with default config if there is no constructor config
     * @param {Input} input
     * @return {String}
     */
    format(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.options.prefix + this.addSeparator() + this.options.suffix;
    }
    /**
     * Unformat the input with default config if there is no constructor config
     * @param {Input} input
     * @return {String}
     */
    unformat(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.realNumber();
    }
}

/**
 * Creates a fuction to clone the objcet
 */
function cloneDeep(data) {
    return JSON.parse(JSON.stringify(data));
}
/**
 * Creates a CustomEvent('input') with detail = { facade: true }
 * used as a way to identify our own input event
 */
function FacadeInputEvent() {
    return new CustomEvent('input', {
        bubbles: true,
        cancelable: true,
        detail: { facade: true }
    });
}
/**
 * Creates a CustomEvent('change') with detail = { facade: true }
 * used as a way to identify our own change event
 */
function FacadeChangeEvent() {
    return new CustomEvent('change', {
        bubbles: true,
        cancelable: true,
        detail: { facade: true }
    });
}
/**
 * ensure that the element we're attaching to is an input element
 * if not try to find an input element in this elements childrens
 */
function getInputElement(el) {
    const inputElement = el instanceof HTMLInputElement ? el : el.querySelector('input');
    /* istanbul ignore next */
    if (!inputElement) {
        throw new Error('facade directive requires an input element');
    }
    return inputElement;
}
/**
 * Updates the cursor position to the right place after the masking rule was applied
 */
function updateCursor(el, position) {
    const setSelectionRange = () => {
        el.setSelectionRange(position, position);
    };
    setSelectionRange();
    // Android Fix
    setTimeout(setSelectionRange(), 1);
}
/**
 * Updates the element's value and unmasked value based on the masking config rules
 *
 * @param {CustomInputElement} el The input element to update
 * @param {object} [options]
 * @param {Boolean} options.emit Wether to dispatch a new InputEvent or not
 * @param {Boolean} options.force Forces the update even if the old value and the new value are the same
 */
function updateValue(el, vnode, { emit = true, force = false, clean = false } = {}) {
    const { options, oldValue } = el;
    const currentValue = vnode && vnode.data && vnode.data.domProps ? vnode.data.domProps.value : el.value;
    if (force || oldValue !== currentValue) {
        const number = new NumberFormat(options).clean(clean && !options.reverseFill);
        let masked = number.format(currentValue);
        let unmasked = number.clean(options && !options.reverseFill).unformat(currentValue);
        // check value with in range max and min value
        if (clean) {
            if (options.max && Number(unmasked) > options.max) {
                masked = number.format(options.max);
                unmasked = number.unformat(options.max);
            }
            else if (options.min && Number(unmasked) < options.min) {
                masked = number.format(options.min);
                unmasked = number.unformat(options.min);
            }
        }
        el.masked = masked;
        el.unmaskedValue = unmasked;
        // safari makes the cursor jump to the end if el.value gets assign even if to the same value
        if (el.value !== masked) {
            el.value = masked;
        }
        // this part needs to be outside the above IF statement for vuetify in firefox
        // drawback is that we endup with two's input events in firefox
        return emit && el.dispatchEvent(FacadeInputEvent());
    }
}
/**
 * Input event handler
 *
 * @param {CustomInputEvent} event The event object
 */
function inputHandler(event, el) {
    const { target, detail } = event;
    // We dont need to run this method on the event we emit (prevent event loop)
    if (detail && detail.facade) {
        return false;
    }
    const { oldValue, options, masked } = target;
    // since we will be emitting our own custom input event
    // we can stop propagation of this native event
    event.stopPropagation();
    let positionFromEnd = target.value.length;
    if (target.selectionEnd) {
        positionFromEnd = target.value.length - target.selectionEnd;
    }
    updateValue(target, null, { emit: false });
    // updated cursor position
    if (options.suffix) {
        positionFromEnd = Math.max(positionFromEnd, options.suffix.length);
    }
    positionFromEnd = target.value.length - positionFromEnd;
    if (options.prefix) {
        positionFromEnd = Math.max(positionFromEnd, options.prefix.length);
    }
    updateCursor(target, positionFromEnd);
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeInputEvent());
    }
}
/**
 * Blur event handler
 */
function blurHandler(event, el) {
    const { target, detail } = event;
    // We dont need to run this method on the event we emit (prevent event loop)
    if (detail && detail.facade) {
        return false;
    }
    const { oldValue, masked } = target;
    updateValue(target, null, { force: true, emit: true, clean: true });
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeChangeEvent());
    }
}
/**
 * Keydown event handler
 */
function keydownHandler(event, el) {
    const { options } = el;
    const regExp = new RegExp(`${options.prefix}|${options.suffix}`, 'g');
    const newValue = el.value.replace(regExp, '');
    const canNegativeInput = options.min < 0 || !options.min;
    if (([110, 190].includes(event.keyCode) || event.key === options.decimal) && newValue.includes(options.decimal)) {
        event.preventDefault();
    }
    else if ([109].includes(event.keyCode) && !canNegativeInput) {
        event.preventDefault();
    }
    else if ([8].includes(event.keyCode)) {
        // check current cursor position is after separator when backspace key down
        const selectionEnd = el.selectionEnd || 0;
        const character = el.value.slice(selectionEnd - 1, selectionEnd);
        const replace = el.value.slice(selectionEnd - 2, selectionEnd);
        if (character === options.separator) {
            event.preventDefault();
            let positionFromEnd = el.value.length - selectionEnd;
            // remove separator and before character
            el.value = el.value.replace(replace, '');
            // updated cursor position
            if (options.suffix) {
                positionFromEnd = Math.max(positionFromEnd, options.suffix.length);
            }
            positionFromEnd = el.value.length - positionFromEnd;
            if (options.prefix) {
                positionFromEnd = Math.max(positionFromEnd, options.prefix.length);
            }
            updateCursor(el, positionFromEnd);
            // trigger input event
            el.dispatchEvent(new Event('input'));
        }
        else if ([options.prefix, '-'].includes(character)) {
            event.preventDefault();
            el.value = '';
            el.dispatchEvent(new Event('input'));
        }
    }
}

var directive = {
    bind: (el, { value, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = Object.assign(cloneDeep(options$1), value, modifiers);
        el.options = options;
        // set initial value
        updateValue(el, vnode, { force: options.prefill, clean: true });
    },
    inserted: (el) => {
        el = getInputElement(el);
        // prefer adding event listener to parent element to avoid Firefox bug which does not
        // execute `useCapture: true` event handlers before non-capturing event handlers
        const handlerOwner = el.parentElement || el;
        // use anonymous event handler to avoid inadvertently removing masking for all inputs within a container
        const oninput = (e) => {
            if (e.target !== el) {
                return;
            }
            inputHandler(e);
        };
        handlerOwner.addEventListener('input', oninput, true);
        el.onblur = (e) => blurHandler(e);
        // check decimal key and insert to current element
        // updated cursor position after format the value
        el.onkeydown = (e) => keydownHandler(e, el);
        el.cleanup = () => handlerOwner.removeEventListener('input', oninput, true);
    },
    update: (el, { value, oldValue, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = el.options;
        el.options = Object.assign(options, value, modifiers);
        if (value !== oldValue) {
            updateValue(el, vnode, { force: true, clean: true });
        }
        else {
            updateValue(el, vnode);
        }
    },
    unbind: (el) => {
        getInputElement(el).cleanup();
    }
};

Vue.directive('number', directive);
const options = cloneDeep(options$1);
var component = Vue.extend({
    name: 'VueNumber',
    props: {
        value: {
            required: true,
            type: [Number, String]
        },
        nullValue: {
            type: [Number, String],
            default: () => options.nullValue
        },
        masked: {
            type: Boolean,
            default: false
        },
        reverseFill: {
            type: Boolean,
            default: options.reverseFill
        },
        prefill: {
            type: Boolean,
            default: options.prefill
        },
        precision: {
            type: Number,
            default: () => options.precision
        },
        minimumFractionDigits: {
            type: Number,
            default: () => options.minimumFractionDigits
        },
        decimal: {
            type: String,
            default: () => options.decimal
        },
        min: {
            type: Number,
            default: () => options.min
        },
        max: {
            type: Number,
            default: () => options.max
        },
        separator: {
            type: String,
            default: () => options.separator
        },
        prefix: {
            type: String,
            default: () => options.prefix
        },
        suffix: {
            type: String,
            default: () => options.suffix
        }
    },
    data() {
        return {
            maskedValue: this.value,
            unmaskedValue: ''
        };
    },
    computed: {
        emittedValue() {
            return this.masked ? this.maskedValue : this.unmaskedValue;
        },
        config() {
            return {
                nullValue: this.nullValue,
                masked: this.masked,
                reverseFill: this.reverseFill,
                prefill: this.prefill,
                precision: this.precision,
                minimumFractionDigits: this.minimumFractionDigits,
                decimal: this.decimal,
                min: this.min,
                max: this.max,
                separator: this.separator,
                prefix: this.prefix,
                suffix: this.suffix
            };
        }
    },
    watch: {
        value(val) {
            if (this.unmaskedValue !== val) {
                this.maskedValue = val;
            }
        }
    },
    methods: {
        input(event) {
            const { target } = event;
            this.maskedValue = target.value;
            this.unmaskedValue = target.unmaskedValue;
            this.$emit('input', this.emittedValue);
        },
        change() {
            this.$emit('change', this.emittedValue);
        }
    },
    template: `<input
    v-number="config"
    type="text"
    autocomplete="off"
    :value="maskedValue"
    class="v-number"
    @change="change"
    @input="input"
    @blur="(evt) => $emit('blur', evt)"
    @focus="(evt) => $emit('focus', evt)"
  >`
});

var index = {
    install(Vue, config) {
        if (config) {
            Object.assign(options$1, config);
        }
        Vue.directive('number', directive);
        Vue.component('VueNumber', component);
    }
};

export { NumberFormat, component, index as default, directive, options$1 as options };
